% Various utility functions that do not belong anywhere else.

\begin{code}

module DarcsUtils ( catchall, ortryrunning, bug, bugDoc,
                    putStrLnError, putDocLnError,
                    withCurrentDirectory, askUser ) where

import System ( ExitCode(..) )
import System.IO ( hFlush, hPutStrLn, stderr, stdout )
import Control.Exception ( bracket )
import Directory ( setCurrentDirectory )
import Workaround ( getCurrentDirectory )
import Monad ( when )
import Printer ( Doc, errorDoc, hPutDocLn, text, ($$) )

catchall :: IO a -> IO a -> IO a
a `catchall` b = a `catch` (\_ -> b)

ortryrunning :: Monad m => m ExitCode -> m ExitCode -> m ExitCode
a `ortryrunning` b = do ret <- a
                        if ret == ExitSuccess
                           then return ret
                           else b

bug :: String -> a
bug s = error $ "bug in darcs!\n" ++ s ++
                "\nPlease report this to bugs@darcs.net"

bugDoc :: Doc -> a
bugDoc s = errorDoc $ text "bug in darcs!"
                   $$ s
                   $$ text "Please report this to bugs@darcs.net"

putStrLnError :: String -> IO ()
putStrLnError = hPutStrLn stderr

putDocLnError :: Doc -> IO ()
putDocLnError = hPutDocLn stderr

withCurrentDirectory :: FilePath -> IO a -> IO a
withCurrentDirectory name m = 
    bracket 
        (do cwd <- getCurrentDirectory
            when (name /= "") (setCurrentDirectory name)
            return cwd)
        (\oldwd -> do setCurrentDirectory oldwd)
        (const m)

askUser :: String -> IO String
askUser prompt = do putStr prompt
                    hFlush stdout
#ifndef WIN32                    
                    getLine
#else
                    getLineStripCr


getLineStripCr :: IO String
getLineStripCr = do line <- getLine
                    let n = (length line) - 1
                    return $ if line!!n == '\r' then (take n line)
                                                else line
#endif
\end{code}
